@@ -73,7 +73,7 @@ static FILE *in_file, *out_file;
 static unsigned char *window;
 static unsigned long *crc_table;
 
-static unsigned long crc = 0xffffffffL;	/* shift register contents */
+static unsigned long crc; /* shift register contents */
 
 /* Return codes from gzip */
 static const int ERROR = 1;
@@ -131,6 +131,8 @@ static void make_crc_table()
 	/* terms of polynomial defining this crc (except x^32): */
 	static int p[] = {0,1,2,4,5,7,8,10,11,12,16,22,23,26};
 
+	/* initial shift register value */
+	crc = 0xffffffffL;	
 	crc_table = (unsigned long *) malloc(256 * sizeof(unsigned long));
 
 	/* Make exclusive-or pattern from polynomial (0xedb88320) */
@@ -388,6 +390,7 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 	unsigned ml, md;			/* masks for bl and bd bits */
 	register unsigned long b;				/* bit buffer */
 	register unsigned k;		/* number of bits in bit buffer */
+	register int input_char;
 
 	/* make local copies of globals */
 	b = bb;					/* initialize bit buffer */
@@ -399,7 +402,9 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 	md = mask_bits[bd];
 	for (;;) {				/* do until end of block */
 		while (k < (unsigned) bl) {
-			b |= ((unsigned long)fgetc(in_file)) << k;
+			input_char = fgetc(in_file);
+			if (input_char == EOF) return 1;
+			b |= ((unsigned long)input_char) << k;
 			k += 8;
 		}
 		if ((e = (t = tl + ((unsigned) b & ml))->e) > 16)
@@ -411,7 +416,9 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 			k -= t->b;
 			e -= 16;
 			while (k < e) {
-				b |= ((unsigned long)fgetc(in_file)) << k;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b |= ((unsigned long)input_char) << k;
 				k += 8;
 			}
 		} while ((e = (t = t->v.t + ((unsigned) b & mask_bits[e]))->e) > 16);
@@ -433,7 +440,9 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 
 			/* get length of block to copy */
 			while (k < e) {
-				b |= ((unsigned long)fgetc(in_file)) << k;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b |= ((unsigned long)input_char) << k;
 				k += 8;
 			}
 			n = t->v.n + ((unsigned) b & mask_bits[e]);
@@ -442,7 +451,9 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 
 			/* decode distance of block to copy */
 			while (k < (unsigned) bd) {
-				b |= ((unsigned long)fgetc(in_file)) << k;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b |= ((unsigned long)input_char) << k;
 				k += 8;
 			}
 
@@ -454,14 +465,18 @@ static int inflate_codes(huft_t *tl, huft_t *td, int bl, int bd)
 					k -= t->b;
 					e -= 16;
 					while (k < e) {
-						b |= ((unsigned long)fgetc(in_file)) << k;
+						input_char = fgetc(in_file);
+						if (input_char == EOF) return 1;
+						b |= ((unsigned long)input_char) << k;
 						k += 8;
 					}
 				} while ((e = (t = t->v.t + ((unsigned) b & mask_bits[e]))->e) > 16);
 			b >>= t->b;
 			k -= t->b;
 			while (k < e) {
-				b |= ((unsigned long)fgetc(in_file)) << k;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b |= ((unsigned long)input_char) << k;
 				k += 8;
 			}
 			d = w - t->v.n - ((unsigned) b & mask_bits[e]);
@@ -529,6 +544,7 @@ static int inflate_block(int *e)
 		7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
 		12, 12, 13, 13
 	};
+	int input_char;
 
 	/* make local bit buffer */
 	b = bb;
@@ -536,7 +552,9 @@ static int inflate_block(int *e)
 
 	/* read in last block bit */
 	while (k < 1) {
-		b |= ((unsigned long)fgetc(in_file)) << k;
+		input_char = fgetc(in_file);
+		if (input_char == EOF) return 1;
+		b |= ((unsigned long)input_char) << k;
 		k += 8;
 	}
 	*e = (int) b & 1;
@@ -545,7 +563,9 @@ static int inflate_block(int *e)
 
 	/* read in block type */
 	while (k < 2) {
-		b |= ((unsigned long)fgetc(in_file)) << k;
+		input_char = fgetc(in_file);
+		if (input_char == EOF) return 1;
+		b |= ((unsigned long)input_char) << k;
 		k += 8;
 	}
 	t = (unsigned) b & 3;
@@ -577,14 +597,18 @@ static int inflate_block(int *e)
 
 			/* get the length and its complement */
 			while (k_stored < 16) {
-				b_stored |= ((unsigned long)fgetc(in_file)) << k_stored;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b_stored |= ((unsigned long)input_char) << k_stored;
 				k_stored += 8;
 			}
 			n = ((unsigned) b_stored & 0xffff);
 			b_stored >>= 16;
 			k_stored -= 16;
 			while (k_stored < 16) {
-				b_stored |= ((unsigned long)fgetc(in_file)) << k_stored;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b_stored |= ((unsigned long)input_char) << k_stored;
 				k_stored += 8;
 			}
 			if (n != (unsigned) ((~b_stored) & 0xffff)) {
@@ -596,7 +620,9 @@ static int inflate_block(int *e)
 			/* read and output the compressed data */
 			while (n--) {
 				while (k_stored < 8) {
-					b_stored |= ((unsigned long)fgetc(in_file)) << k_stored;
+					input_char = fgetc(in_file);
+					if (input_char == EOF) return 1;
+					b_stored |= ((unsigned long)input_char) << k_stored;
 					k_stored += 8;
 				}
 				window[w++] = (unsigned char) b_stored;
@@ -697,21 +723,27 @@ static int inflate_block(int *e)
 
 			/* read in table lengths */
 			while (k_dynamic < 5) {
-				b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 				k_dynamic += 8;
 			}
 			nl = 257 + ((unsigned) b_dynamic & 0x1f);	/* number of literal/length codes */
 			b_dynamic >>= 5;
 			k_dynamic -= 5;
 			while (k_dynamic < 5) {
-				b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 				k_dynamic += 8;
 			}
 			nd = 1 + ((unsigned) b_dynamic & 0x1f);	/* number of distance codes */
 			b_dynamic >>= 5;
 			k_dynamic -= 5;
 			while (k_dynamic < 4) {
-				b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+				input_char = fgetc(in_file);
+				if (input_char == EOF) return 1;
+				b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 				k_dynamic += 8;
 			}
 			nb = 4 + ((unsigned) b_dynamic & 0xf);	/* number of bit length codes */
@@ -724,7 +756,9 @@ static int inflate_block(int *e)
 			/* read in bit-length-code lengths */
 			for (j = 0; j < nb; j++) {
 				while (k_dynamic < 3) {
-					b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+					input_char = fgetc(in_file);
+					if (input_char == EOF) return 1;
+					b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 					k_dynamic += 8;
 				}
 				ll[border[j]] = (unsigned) b_dynamic & 7;
@@ -750,7 +784,9 @@ static int inflate_block(int *e)
 			i = l = 0;
 			while ((unsigned) i < n) {
 				while (k_dynamic < (unsigned) bl) {
-					b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+					input_char = fgetc(in_file);
+					if (input_char == EOF) return 1;
+					b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 					k_dynamic += 8;
 				}
 				j = (td = tl + ((unsigned) b_dynamic & m))->b;
@@ -762,7 +798,9 @@ static int inflate_block(int *e)
 				}
 				else if (j == 16) {		/* repeat last length 3 to 6 times */
 					while (k_dynamic < 2) {
-						b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+						input_char = fgetc(in_file);
+						if (input_char == EOF) return 1;
+						b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 						k_dynamic += 8;
 					}
 					j = 3 + ((unsigned) b_dynamic & 3);
@@ -776,7 +814,9 @@ static int inflate_block(int *e)
 					}
 				} else if (j == 17) {	/* 3 to 10 zero length codes */
 					while (k_dynamic < 3) {
-						b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+						input_char = fgetc(in_file);
+						if (input_char == EOF) return 1;
+						b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 						k_dynamic += 8;
 					}
 					j = 3 + ((unsigned) b_dynamic & 7);
@@ -791,7 +831,9 @@ static int inflate_block(int *e)
 					l = 0;
 				} else {		/* j == 18: 11 to 138 zero length codes */
 					while (k_dynamic < 7) {
-						b_dynamic |= ((unsigned long)fgetc(in_file)) << k_dynamic;
+						input_char = fgetc(in_file);
+						if (input_char == EOF) return 1;
+						b_dynamic |= ((unsigned long)input_char) << k_dynamic;
 						k_dynamic += 8;
 					}
 					j = 11 + ((unsigned) b_dynamic & 0x7f);
@@ -875,6 +917,13 @@ static int inflate()
 		}
 	} while (!e);
 
+	/* Undo too much lookahead.  The next read will be byte aligned so we
+	 * can discard unused bits in the last meaningful byte.  */
+	while (bk >= 8) {
+		bk -= 8;
+		ungetc((bb << bk), in_file);
+	}
+
 	/* flush out window */
 	flush_window();
 
@@ -935,7 +984,7 @@ extern int unzip(FILE *l_in_file, FILE *l_out_file)
 	}
 
 	method = (int) fgetc(in_file);
-	if (method != 8) {
+	if (method != 8) {                   /* also catches EOF */
 		error_msg("unknown method %d -- get newer version of gzip", method);
 		exit_code = 1;
 		return -1;
@@ -951,6 +1000,7 @@ extern int unzip(FILE *l_in_file, FILE *l_out_file)
 		size_t extra;
 		extra = fgetc(in_file);
 		extra += fgetc(in_file) << 8;
+		if (feof(in_file)) return 1;
 
 		for (i = 0; i < extra; i++)
 			fgetc(in_file);
@@ -958,12 +1008,12 @@ extern int unzip(FILE *l_in_file, FILE *l_out_file)
 
 	/* Discard original name if any */
 	if ((flags & orig_name) != 0) {
-		while (fgetc(in_file) != 0);	/* null */
+		while (fgetc(in_file) != 0 && !feof(in_file));	/* null */
 	}
 
 	/* Discard file comment if any */
 	if ((flags & comment) != 0) {
-		while (fgetc(in_file) != 0);	/* null */
+		while (fgetc(in_file) != 0 && !feof(in_file));	/* null */
 	}
 
 	if (method < 0) {
@@ -980,12 +1030,15 @@ extern int unzip(FILE *l_in_file, FILE *l_out_file)
 
 		if (res == 3) {
 			error_msg(memory_exhausted);
+			exit_code = 1;
 		} else if (res != 0) {
 			error_msg("invalid compressed data--format violated");
+			exit_code = 1;
 		}
 
 	} else {
 		error_msg("internal error, invalid method");
+		exit_code = 1;
 	}
 
 	/* Get the crc and original length
@@ -995,18 +1048,20 @@ extern int unzip(FILE *l_in_file, FILE *l_out_file)
 	fread(buf, 1, 8, in_file);
 
 	/* Validate decompression - crc */
-	if ((unsigned int)((buf[0] | (buf[1] << 8)) |((buf[2] | (buf[3] << 8)) << 16)) != (crc ^ 0xffffffffL)) {
+	if (!exit_code && (unsigned int)((buf[0] | (buf[1] << 8)) |((buf[2] | (buf[3] << 8)) << 16)) != (crc ^ 0xffffffffL)) {
 		error_msg("invalid compressed data--crc error");
+		exit_code = 1;
 	}
 	/* Validate decompression - size */
-	if (((buf[4] | (buf[5] << 8)) |((buf[6] | (buf[7] << 8)) << 16)) != (unsigned long) bytes_out) {
+	if (!exit_code && ((buf[4] | (buf[5] << 8)) |((buf[6] | (buf[7] << 8)) << 16)) != (unsigned long) bytes_out) {
 		error_msg("invalid compressed data--length error");
+		exit_code = 1;
 	}
 
 	free(window);
 	free(crc_table);
 
-	return 0;
+	return exit_code;
 }
 
 /*
